@@ -23,14 +23,11 @@ module Hakyll.Web.Feed
     ) where
 
 
---------------------------------------------------------------------------------
-import           Control.Monad                 ((<=<))
-
-
 --------------------------------------------------------------------------------
 import           Hakyll.Core.Compiler
 import           Hakyll.Core.Compiler.Internal
 import           Hakyll.Core.Item
+import           Hakyll.Core.Util.String (replaceAll)
 import           Hakyll.Web.Template
 import           Hakyll.Web.Template.Context
 import           Hakyll.Web.Template.List
@@ -65,14 +62,21 @@ renderFeed :: FilePath                -- ^ Feed template
            -> [Item String]           -- ^ Input items
            -> Compiler (Item String)  -- ^ Resulting item
 renderFeed feedPath itemPath config itemContext items = do
-    feedTpl <- compilerUnsafeIO $ loadTemplate feedPath
-    itemTpl <- compilerUnsafeIO $ loadTemplate itemPath
+    feedTpl <- loadTemplate feedPath
+    itemTpl <- loadTemplate itemPath
 
-    body <- makeItem =<< applyTemplateList itemTpl itemContext' items
+    protectedItems <- mapM (applyFilter protectCDATA) items
+    body <- makeItem =<< applyTemplateList itemTpl itemContext' protectedItems
     applyTemplate feedTpl feedContext body
   where
+    applyFilter :: (Monad m,Functor f) => (String -> String) -> f String -> m (f String)
+    applyFilter tr str = return $ fmap tr str
+    protectCDATA :: String -> String
+    protectCDATA = replaceAll "]]>" (const "]]&gt;")
     -- Auxiliary: load a template from a datafile
-    loadTemplate = fmap readTemplate . readFile <=< getDataFileName
+    loadTemplate path = do
+        file <- compilerUnsafeIO $ getDataFileName path
+        unsafeReadTemplateFile file
 
     itemContext' = mconcat
         [ itemContext
